import json
import logging

from .model import APIModel, APIEndpoints, RequestsMethods, CustomRole
from .api import Api


class RBAC:
    """The class includes all necessary methods to access the Grafana RBAC API endpoints. Be aware that the functionality is a Grafana ENTERPRISE feature

    Args:
        grafana_api_model (APIModel): Inject a Grafana API model object that includes all necessary values and information

    Attributes:
        grafana_api_model (APIModel): This is where we store the grafana_api_model
    """

    def __init__(self, grafana_api_model: APIModel):
        self.grafana_api_model = grafana_api_model

    def get_status(self) -> bool:
        """The method includes a functionality to get the status if role-based access control is enabled or not

        Required Permissions:
            Action: status:accesscontrol
            Scope: services:accesscontrol

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (bool): Return a flag indicating if the role-based access control is enabled or not
        """

        api_call: dict = Api(self.grafana_api_model).call_the_api(
            f"{APIEndpoints.RBAC.value}/status", response_status_code=True
        )

        status_code: int = api_call.get("status")

        alert_manager_status_dict: dict = dict(
            {
                403: "Access denied.",
                404: "Not found, an indication that role-based access control is not available at all.",
                500: "Unexpected error. Refer to body and/or server logs for more details.",
            }
        )

        if status_code == 200:
            return bool(api_call.get("enabled"))
        elif 403 <= status_code <= 500:
            logging.error(alert_manager_status_dict.get(status_code))
            raise Exception
        else:
            logging.error(f"Check the error: {api_call}.")
            raise Exception

    def get_all_roles(self, include_hidden_roles: bool = False) -> list:
        """The method includes a functionality gets all existing roles. The response contains all global and organization local roles, for the organization which user is signed in

        Args:
            include_hidden_roles (bool): Specify if the output contains the hidden roles or not (default False)

        Required Permissions:
            Action: roles:read
            Scope: roles:*

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (list): Return all global and organization local roles
        """

        additional_parameters: str = ""
        if include_hidden_roles:
            additional_parameters = "?includeHidden=true"

        api_call: list = Api(self.grafana_api_model).call_the_api(
            f"{APIEndpoints.RBAC.value}/roles{additional_parameters}",
            response_status_code=True,
        )

        status_code: int = api_call[0].get("status")

        alert_manager_status_dict: dict = dict(
            {
                403: "Access denied.",
                500: "Unexpected error. Refer to body and/or server logs for more details.",
            }
        )

        if status_code == 200:
            return api_call
        elif 403 <= status_code <= 500:
            logging.error(alert_manager_status_dict.get(status_code))
            raise Exception
        else:
            logging.error(f"Check the error: {api_call}.")
            raise Exception

    def get_role(self, uid: str) -> dict:
        """The method includes a functionality get a role specified by the uid

        Args:
            uid (str): Specify the uid of the role

        Required Permissions:
            Action: roles:read
            Scope: roles:*

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Return the corresponding role
        """

        if len(uid) != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.RBAC.value}/roles/{uid}", response_status_code=True
            )

            status_code: int = api_call.get("status")

            alert_manager_status_dict: dict = dict(
                {
                    403: "Access denied.",
                    500: "Unexpected error. Refer to body and/or server logs for more details.",
                }
            )

            if status_code == 200:
                return api_call
            elif 403 <= status_code <= 500:
                logging.error(alert_manager_status_dict.get(status_code))
                raise Exception
            else:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
        else:
            logging.error("There is no uid defined.")
            raise ValueError

    def create_role(self, role_definition: CustomRole) -> dict:
        """The method includes a functionality create a new custom role and maps given permissions to that role. Note that roles with the same prefix as Fixed roles can’t be created

        Args:
            role_definition (CustomRole): Specify the corresponding role definition

        Required Permissions:
            Action: roles:write
            Scope: permissions:type:delegate

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Return the newly created role
        """

        if role_definition is not None and len(role_definition.name) != 0:
            role_object: dict = dict(
                {
                    "name": role_definition.name,
                    "global": role_definition.global_role,
                    "hidden": role_definition.hidden,
                }
            )

            if role_definition.uid is not None:
                role_object.update(dict({"uid": role_definition.uid}))

            if role_definition.version is not None:
                role_object.update(dict({"version": role_definition.version}))

            if role_definition.description is not None:
                role_object.update(dict({"description": role_definition.description}))

            if role_definition.display_name is not None:
                role_object.update(dict({"displayName": role_definition.display_name}))

            if role_definition.group is not None:
                role_object.update(dict({"group": role_definition.group}))

            if role_definition.permissions is not None:
                permission_list: list = list()
                for permission in role_definition.permissions:
                    permission_object: dict = dict()
                    if permission.action is not None:
                        permission_object.update({"action": permission.action})
                    else:
                        logging.error("There is no permission action value defined.")
                        raise ValueError

                    if permission.scope is not None:
                        permission_object.update({"scope": permission.scope})

                    permission_list.append(permission_object)
                role_object.update(dict({"permissions": permission_list}))

            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.RBAC.value}/roles",
                RequestsMethods.POST,
                json.dumps(role_object),
                response_status_code=True,
            )

            status_code: int = api_call.get("status")

            alert_manager_status_dict: dict = dict(
                {
                    400: "Bad request (invalid json, missing content-type, missing or invalid fields, etc.).",
                    403: "Access denied.",
                    500: "Unexpected error. Refer to body and/or server logs for more details.",
                }
            )

            if status_code == 200:
                return api_call
            elif 400 <= status_code <= 500:
                logging.error(alert_manager_status_dict.get(status_code))
                raise Exception
            else:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
        else:
            logging.error("There is no role_definition or name defined.")
            raise ValueError

    def update_role(self, uid: str, role_definition: CustomRole) -> dict:
        """The method includes a functionality to update the role with the given uid, and its permissions. The operation is idempotent and all permissions of the role will be replaced based on the request content. You need to increment the version of the role with each update, otherwise the request will fail

        Args:
            uid (str): Specify the corresponding uid of the custom role
            role_definition (CustomRole): Specify the corresponding role definition

        Required Permissions:
            Action: roles:write
            Scope: permissions:type:delegate

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Return the updated role
        """

        if (
            role_definition is not None
            and len(role_definition.name) != 0
            and role_definition.version != 0
            and len(uid) != 0
        ):
            role_object: dict = dict(
                {
                    "name": role_definition.name,
                    "global": role_definition.global_role,
                    "hidden": role_definition.hidden,
                    "version": role_definition.version,
                }
            )

            if role_definition.description is not None:
                role_object.update(dict({"description": role_definition.description}))

            if role_definition.display_name is not None:
                role_object.update(dict({"displayName": role_definition.display_name}))

            if role_definition.group is not None:
                role_object.update(dict({"group": role_definition.group}))

            if role_definition.permissions is not None:
                permission_list: list = list()
                for permission in role_definition.permissions:
                    permission_object: dict = dict()
                    if permission.action is not None:
                        permission_object.update({"action": permission.action})
                    else:
                        logging.error("There is no permission action value defined.")
                        raise ValueError

                    if permission.scope is not None:
                        permission_object.update({"scope": permission.scope})

                    permission_list.append(permission_object)
                role_object.update(dict({"permissions": permission_list}))

            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.RBAC.value}/roles/{uid}",
                RequestsMethods.PUT,
                json.dumps(role_object),
                response_status_code=True,
            )

            status_code: int = api_call.get("status")

            alert_manager_status_dict: dict = dict(
                {
                    400: "Bad request (invalid json, missing content-type, missing or invalid fields, etc.).",
                    403: "Access denied.",
                    404: "Role was not found to update.",
                    500: "Unexpected error. Refer to body and/or server logs for more details.",
                }
            )

            if status_code == 200:
                return api_call
            elif 400 <= status_code <= 500:
                logging.error(alert_manager_status_dict.get(status_code))
                raise Exception
            else:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
        else:
            logging.error("There is no role_definition, version, name or uid defined.")
            raise ValueError

    def delete_role(self, uid: str, force: bool = False, global_role: bool = False):
        """The method includes a functionality to delete a role with the given uid

        Args:
            uid (str): Specify the corresponding uid of the custom role
            force (bool): Specify the corresponding if the role will be deleted with all it’s assignments or not (default False)
            global_role (bool): Specify the corresponding if the role is global or not. If set to false, the default org id of the authenticated user will be used from the request (default False)

        Required Permissions:
            Action: roles:delete
            Scope: permissions:type:delegate

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if len(uid) != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.RBAC.value}/roles/{uid}?force={force.__str__().lower()}&"
                f"global={global_role.__str__().lower()}",
                RequestsMethods.DELETE,
                response_status_code=True,
            )

            status_code: int = api_call.get("status")

            alert_manager_status_dict: dict = dict(
                {
                    400: "Bad request (invalid json, missing content-type, missing or invalid fields, etc.).",
                    403: "Access denied.",
                    500: "Unexpected error. Refer to body and/or server logs for more details.",
                }
            )

            if status_code == 200:
                if "Role deleted" != api_call.get("message"):
                    logging.error(f"Please, check the error: {api_call}.")
                    raise Exception
                else:
                    logging.info("You successfully destroyed the role.")
            elif 400 <= status_code <= 500:
                logging.error(alert_manager_status_dict.get(status_code))
                raise Exception
            else:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
        else:
            logging.error("There is no uid defined.")
            raise ValueError

    def get_user_assigned_roles(
        self, user_id: int, include_hidden_roles: bool = False
    ) -> list:
        """The method includes a functionality to get the roles that have been directly assigned to a given user specified by the user id. The list does not include basic roles (Viewer, Editor, Admin or Grafana Admin), and it does not include roles that have been inherited from a team

        Args:
            user_id (int): Specify the corresponding user_id of the user
            include_hidden_roles (bool): Specify if the output contains the hidden roles or not (default False)

        Required Permissions:
            Action: users.roles:read
            Scope: users:id:<user ID>

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (list): Return the corresponding user roles
        """

        if user_id != 0 and user_id is not None:
            additional_parameters: str = ""
            if include_hidden_roles:
                additional_parameters = "?includeHidden=true"

            api_call: list = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.RBAC.value}/users/{user_id}/roles{additional_parameters}",
                response_status_code=True,
            )

            status_code: int = api_call[0].get("status")

            alert_manager_status_dict: dict = dict(
                {
                    403: "Access denied.",
                    500: "Unexpected error. Refer to body and/or server logs for more details.",
                }
            )

            if status_code == 200:
                return api_call
            elif 403 <= status_code <= 500:
                logging.error(alert_manager_status_dict.get(status_code))
                raise Exception
            else:
                logging.error(f"Check the error: {api_call}.")
            raise Exception
        else:
            logging.error("There is no user_id defined.")
            raise ValueError

    def get_user_assigned_permissions(self, user_id: int) -> list:
        """The method includes a functionality to get the permissions that have been directly assigned to a given user specified by the user id

        Args:
            user_id (int): Specify the corresponding user_id of the user

        Required Permissions:
            Action: users.permissions:read
            Scope: users:id:<user ID>

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (list): Return the corresponding user permissions
        """

        if user_id != 0 and user_id is not None:
            api_call: list = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.RBAC.value}/users/{user_id}/permissions",
                response_status_code=True,
            )

            status_code: int = api_call[0].get("status")

            alert_manager_status_dict: dict = dict(
                {
                    403: "Access denied.",
                    500: "Unexpected error. Refer to body and/or server logs for more details.",
                }
            )

            if status_code == 200:
                return api_call
            elif 403 <= status_code <= 500:
                logging.error(alert_manager_status_dict.get(status_code))
                raise Exception
            else:
                logging.error(f"Check the error: {api_call}.")
            raise Exception
        else:
            logging.error("There is no user_id defined.")
            raise ValueError

    def add_user_role_assignment(
        self, user_id: int, role_uid: str, global_assignment: bool = False
    ):
        """The method includes a functionality to assign a role to a specific user

        Args:
            user_id (int): Specify the corresponding user_id of the user
            role_uid (str): Specify the corresponding uid of the role
            global_assignment (bool): Specify the corresponding if the assignment is global or not. If set to false, the default org id of the authenticated user will be used from the request (default False)

        Required Permissions:
            Action: users.roles:add
            Scope: permissions:type:delegate

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if user_id != 0 and user_id is not None and len(role_uid) != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.RBAC.value}/users/{user_id}/roles",
                RequestsMethods.POST,
                json.dumps({"global": global_assignment, "roleUid": role_uid}),
                response_status_code=True,
            )

            status_code: int = api_call.get("status")

            alert_manager_status_dict: dict = dict(
                {
                    403: "Access denied.",
                    404: "Role not found.",
                    500: "Unexpected error. Refer to body and/or server logs for more details.",
                }
            )

            if status_code == 200:
                if "Role added to the user." != api_call.get("message"):
                    logging.error(f"Please, check the error: {api_call}.")
                    raise Exception
                else:
                    logging.info("You successfully added the role of the user.")
            elif 403 <= status_code <= 500:
                logging.error(alert_manager_status_dict.get(status_code))
                raise Exception
            else:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
        else:
            logging.error("There is no user_id or role_uid defined.")
            raise ValueError

    def remove_user_role_assignment(self, user_id: int, role_uid: str):
        """The method includes a functionality to revoke a role to a specific user

        Args:
            user_id (int): Specify the corresponding user_id of the user
            role_uid (str): Specify the corresponding uid of the role

        Required Permissions:
            Action: users.roles:remove
            Scope: permissions:type:delegate

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if user_id != 0 and user_id is not None and len(role_uid) != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.RBAC.value}/users/{user_id}/roles/{role_uid}",
                RequestsMethods.DELETE,
                response_status_code=True,
            )

            status_code: int = api_call.get("status")

            alert_manager_status_dict: dict = dict(
                {
                    403: "Access denied.",
                    500: "Unexpected error. Refer to body and/or server logs for more details.",
                }
            )

            if status_code == 200:
                if "Role removed from user." != api_call.get("message"):
                    logging.error(f"Please, check the error: {api_call}.")
                    raise Exception
                else:
                    logging.info("You successfully removed the role of the user.")
            elif 403 <= status_code <= 500:
                logging.error(alert_manager_status_dict.get(status_code))
                raise Exception
            else:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
        else:
            logging.error("There is no user_id or role_uid defined.")
            raise ValueError

    def update_user_role_assignments(
        self,
        user_id: int,
        role_uids: list,
        include_hidden_roles: bool = False,
        global_assignment: bool = False,
    ):
        """The method includes a functionality to update the user role assignments to match the provided set of uid's. This will remove any assigned roles that aren’t in the request and add roles that are in the set but are not already assigned to the user

        Args:
            user_id (int): Specify the corresponding user_id of the user
            role_uids (list): Specify the corresponding uids of the role
            include_hidden_roles (bool): Specify if the output contains the hidden roles or not (default False)
            global_assignment (bool): Specify the corresponding if the assignment is global or not. If set to false, the default org id of the authenticated user will be used from the request (default False)

        Required Permissions:
            Action: users.roles:add, users.roles:remove
            Scope: permissions:type:delegate

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if user_id != 0 and user_id is not None and len(role_uids) != 0:
            additional_parameters: str = ""
            if include_hidden_roles:
                additional_parameters = "?includeHidden=true"

            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.RBAC.value}/users/{user_id}/roles{additional_parameters}",
                RequestsMethods.PUT,
                json.dumps({"global": global_assignment, "roleUids": role_uids}),
                response_status_code=True,
            )

            status_code: int = api_call.get("status")

            alert_manager_status_dict: dict = dict(
                {
                    403: "Access denied.",
                    404: "Role not found.",
                    500: "Unexpected error. Refer to body and/or server logs for more details.",
                }
            )

            if status_code == 200:
                if "User roles have been updated." != api_call.get("message"):
                    logging.error(f"Please, check the error: {api_call}.")
                    raise Exception
                else:
                    logging.info("You successfully updated the roles of the user.")
            elif 403 <= status_code <= 500:
                logging.error(alert_manager_status_dict.get(status_code))
                raise Exception
            else:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
        else:
            logging.error("There is no user_id or role_uids defined.")
            raise ValueError

    def get_service_account_assigned_roles(
        self, service_account_id: int, include_hidden_roles: bool = False
    ) -> list:
        """The method includes a functionality to get the roles that have been directly assigned to a given service account. The list does not include basic roles (Viewer, Editor, Admin or Grafana Admin)

        Args:
            service_account_id (int): Specify the corresponding service_account_id of the service account
            include_hidden_roles (bool): Specify if the output contains the hidden roles or not (default False)

        Required Permissions:
            Action: users.roles:read
            Scope: users:id:<service account ID>

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (list): Return the corresponding service account roles
        """

        if service_account_id != 0 and service_account_id is not None:
            additional_parameters: str = ""
            if include_hidden_roles:
                additional_parameters = "?includeHidden=true"

            api_call: list = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.RBAC.value}/users/{service_account_id}/roles{additional_parameters}",
                response_status_code=True,
            )

            status_code: int = api_call[0].get("status")

            alert_manager_status_dict: dict = dict(
                {
                    403: "Access denied.",
                    500: "Unexpected error. Refer to body and/or server logs for more details.",
                }
            )

            if status_code == 200:
                return api_call
            elif 403 <= status_code <= 500:
                logging.error(alert_manager_status_dict.get(status_code))
                raise Exception
            else:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
        else:
            logging.error("There is no service_account_id defined.")
            raise ValueError

    def get_service_account_assigned_permissions(self, service_account_id: int) -> list:
        """The method includes a functionality to get the permissions that a given service account has

        Args:
            service_account_id (int): Specify the corresponding service_account_id of the service account

        Required Permissions:
            Action: users.permissions:read
            Scope: users:id:<service account ID>

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (list): Return the corresponding service account permissions
        """

        if service_account_id != 0 and service_account_id is not None:
            api_call: list = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.RBAC.value}/users/{service_account_id}/permissions",
                response_status_code=True,
            )

            status_code: int = api_call[0].get("status")

            alert_manager_status_dict: dict = dict(
                {
                    403: "Access denied.",
                    500: "Unexpected error. Refer to body and/or server logs for more details.",
                }
            )

            if status_code == 200:
                return api_call
            elif 403 <= status_code <= 500:
                logging.error(alert_manager_status_dict.get(status_code))
                raise Exception
            else:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
        else:
            logging.error("There is no service_account_id defined.")
            raise ValueError

    def add_service_account_role_assignment(
        self, service_account_id: int, role_uid: str, global_assignment: bool = False
    ):
        """The method includes a functionality to assign a role to a specific service account

        Args:
            service_account_id (int): Specify the corresponding service_account_id of the service account
            role_uid (str): Specify the corresponding uid of the role
            global_assignment (bool): Specify the corresponding if the assignment is global or not. If set to false, the default org id of the authenticated user will be used from the request (default False)

        Required Permissions:
            Action: users.roles:add
            Scope: permissions:type:delegate

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if (
            service_account_id != 0
            and service_account_id is not None
            and len(role_uid) != 0
        ):
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.RBAC.value}/users/{service_account_id}/roles",
                RequestsMethods.POST,
                json.dumps({"global": global_assignment, "roleUid": role_uid}),
                response_status_code=True,
            )

            status_code: int = api_call.get("status")
            alert_manager_status_dict: dict = dict(
                {
                    403: "Access denied.",
                    404: "Role not found.",
                    500: "Unexpected error. Refer to body and/or server logs for more details.",
                }
            )

            if status_code == 200:
                if "Role added to the user." != api_call.get("message"):
                    logging.error(f"Please, check the error: {api_call}.")
                    raise Exception
                else:
                    logging.info(
                        "You successfully added the role of the service account."
                    )
            elif 403 <= status_code <= 500:
                logging.error(alert_manager_status_dict.get(status_code))
                raise Exception
            else:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
        else:
            logging.error("There is no service_account_id or role_uid defined.")
            raise ValueError

    def remove_service_account_role_assignment(
        self, service_account_id: int, role_uid: str
    ):
        """The method includes a functionality to revoke a role to a specific service account

        Args:
            service_account_id (int): Specify the corresponding service_account_id of the service account
            role_uid (str): Specify the corresponding uid of the role

        Required Permissions:
            Action: users.roles:remove
            Scope: permissions:type:delegate

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if (
            service_account_id != 0
            and service_account_id is not None
            and len(role_uid) != 0
        ):
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.RBAC.value}/users/{service_account_id}/roles/{role_uid}",
                RequestsMethods.DELETE,
                response_status_code=True,
            )

            status_code: int = api_call.get("status")

            alert_manager_status_dict: dict = dict(
                {
                    403: "Access denied.",
                    500: "Unexpected error. Refer to body and/or server logs for more details.",
                }
            )

            if status_code == 200:
                if "Role removed from user." != api_call.get("message"):
                    logging.error(f"Please, check the error: {api_call}.")
                    raise Exception
                else:
                    logging.info(
                        "You successfully removed the role of the service account."
                    )
            elif 403 <= status_code <= 500:
                logging.error(alert_manager_status_dict.get(status_code))
                raise Exception
            else:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
        else:
            logging.error("There is no service_account_id or role_uid defined.")
            raise ValueError

    def update_service_account_role_assignments(
        self,
        service_account_id: int,
        role_uids: list,
        include_hidden_roles: bool = False,
        global_assignment: bool = False,
    ):
        """The method includes a functionality to update the service account role assignments to match the provided set of uid's. This will remove any assigned roles that aren’t in the request and add roles that are in the set but are not already assigned to the user

        Args:
            service_account_id (int): Specify the corresponding service_account_id of the service account
            role_uids (list): Specify the corresponding uids of the role
            include_hidden_roles (bool): Specify if the output contains the hidden roles or not (default False)
            global_assignment (bool): Specify the corresponding if the assignment is global or not. If set to false, the default org id of the authenticated user will be used from the request (default False)

        Required Permissions:
            Action: users.roles:add, users.roles:remove
            Scope: permissions:type:delegate

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if (
            service_account_id != 0
            and service_account_id is not None
            and len(role_uids) != 0
        ):
            additional_parameters: str = ""
            if include_hidden_roles:
                additional_parameters = "?includeHidden=true"

            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.RBAC.value}/users/{service_account_id}/roles{additional_parameters}",
                RequestsMethods.PUT,
                json.dumps({"global": global_assignment, "roleUids": role_uids}),
                response_status_code=True,
            )

            status_code: int = api_call.get("status")

            alert_manager_status_dict: dict = dict(
                {
                    403: "Access denied.",
                    404: "Role not found.",
                    500: "Unexpected error. Refer to body and/or server logs for more details.",
                }
            )

            if status_code == 200:
                if "User roles have been updated." != api_call.get("message"):
                    logging.error(f"Please, check the error: {api_call}.")
                    raise Exception
                else:
                    logging.info(
                        "You successfully updated the roles of the service account."
                    )
            elif 403 <= status_code <= 500:
                logging.error(alert_manager_status_dict.get(status_code))
                raise Exception
            else:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
        else:
            logging.error("There is no service_account_id or role_uids defined.")
            raise ValueError

    def get_team_assigned_roles(
        self, team_id: int, include_hidden_roles: bool = False
    ) -> list:
        """The method includes a functionality to get that have been directly assigned to a given team.

        Args:
            team_id (int): Specify the corresponding team_id of the team
            include_hidden_roles (bool): Specify if the output contains the hidden roles or not (default False)

        Required Permissions:
            Action: teams.roles:read
            Scope: teams:id:<team ID>

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (list): Return the corresponding team roles
        """

        if team_id != 0 and team_id is not None:
            additional_parameters: str = ""
            if include_hidden_roles:
                additional_parameters = "?includeHidden=true"

            api_call: list = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.RBAC.value}/teams/{team_id}/roles{additional_parameters}",
                response_status_code=True,
            )

            status_code: int = api_call[0].get("status")

            alert_manager_status_dict: dict = dict(
                {
                    403: "Access denied.",
                    500: "Unexpected error. Refer to body and/or server logs for more details.",
                }
            )

            if status_code == 200:
                return api_call
            elif 403 <= status_code <= 500:
                logging.error(alert_manager_status_dict.get(status_code))
                raise Exception
            else:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
        else:
            logging.error("There is no team_id defined.")
            raise ValueError

    def add_team_role_assignment(self, team_id: int, role_uid: str):
        """The method includes a functionality to assign a role to a specific team

        Args:
            team_id (int): Specify the corresponding team_id of the team
            role_uid (str): Specify the corresponding uid of the role

        Required Permissions:
            Action: teams.roles:add
            Scope: permissions:type:delegate

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if team_id != 0 and team_id is not None and len(role_uid) != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.RBAC.value}/teams/{team_id}/roles",
                RequestsMethods.POST,
                json.dumps({"roleUid": role_uid}),
                response_status_code=True,
            )

            status_code: int = api_call.get("status")

            alert_manager_status_dict: dict = dict(
                {
                    403: "Access denied.",
                    404: "Role not found.",
                    500: "Unexpected error. Refer to body and/or server logs for more details.",
                }
            )

            if status_code == 200:
                if "Role added to the team." != api_call.get("message"):
                    logging.error(f"Please, check the error: {api_call}.")
                    raise Exception
                else:
                    logging.info("You successfully added the role of the team.")
            elif 403 <= status_code <= 500:
                logging.error(alert_manager_status_dict.get(status_code))
                raise Exception
            else:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
        else:
            logging.error("There is no team_id or role_uid defined.")
            raise ValueError

    def remove_team_role_assignment(self, team_id: int, role_uid: str):
        """The method includes a functionality to revoke a role to a specific team

        Args:
            team_id (int): Specify the corresponding team_id of the team
            role_uid (str): Specify the corresponding uid of the role

        Required Permissions:
            Action: teams.roles:remove
            Scope: permissions:type:delegate

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if team_id != 0 and team_id is not None and len(role_uid) != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.RBAC.value}/teams/{team_id}/roles/{role_uid}",
                RequestsMethods.DELETE,
                response_status_code=True,
            )

            status_code: int = api_call.get("status")

            alert_manager_status_dict: dict = dict(
                {
                    403: "Access denied.",
                    500: "Unexpected error. Refer to body and/or server logs for more details.",
                }
            )

            if status_code == 200:
                if "Role removed from team." != api_call.get("message"):
                    logging.error(f"Please, check the error: {api_call}.")
                    raise Exception
                else:
                    logging.info("You successfully removed the role of the team.")
            elif 403 <= status_code <= 500:
                logging.error(alert_manager_status_dict.get(status_code))
                raise Exception
            else:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
        else:
            logging.error("There is no team_id or role_uid defined.")
            raise ValueError

    def update_team_role_assignments(
        self,
        team_id: int,
        role_uids: list,
        include_hidden_roles: bool = False,
    ):
        """The method includes a functionality to update the service account role assignments to match the provided set of uid's. This will remove any assigned roles that aren’t in the request and add roles that are in the set but are not already assigned to the user

        Args:
            team_id (int): Specify the corresponding team_id of the team
            role_uids (list): Specify the corresponding uids of the role
            include_hidden_roles (bool): Specify if the output contains the hidden roles or not (default False)

        Required Permissions:
            Action: teams.roles:add, teams.roles:remove
            Scope: permissions:type:delegate

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if team_id != 0 and team_id is not None and len(role_uids) != 0:
            additional_parameters: str = ""
            if include_hidden_roles:
                additional_parameters = "?includeHidden=true"

            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.RBAC.value}/teams/{team_id}/roles{additional_parameters}",
                RequestsMethods.PUT,
                json.dumps({"roleUids": role_uids}),
                response_status_code=True,
            )

            status_code: int = api_call.get("status")

            alert_manager_status_dict: dict = dict(
                {
                    403: "Access denied.",
                    404: "Role not found.",
                    500: "Unexpected error. Refer to body and/or server logs for more details.",
                }
            )

            if status_code == 200:
                if "Team roles have been updated." != api_call.get("message"):
                    logging.error(f"Please, check the error: {api_call}.")
                    raise Exception
                else:
                    logging.info("You successfully updated the roles of the team.")
            elif 403 <= status_code <= 500:
                logging.error(alert_manager_status_dict.get(status_code))
                raise Exception
            else:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
        else:
            logging.error("There is no team_id or role_uids defined.")
            raise ValueError

    def reset_basic_roles_to_their_default(self):
        """The method includes a functionality to reset basic roles permissions to their default

        Required Permissions:
            Action: roles:write
            Scope: permissions:type:escalate

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        api_call: dict = Api(self.grafana_api_model).call_the_api(
            f"{APIEndpoints.RBAC.value}/roles/hard-reset",
            RequestsMethods.POST,
            json.dumps({"BasicRoles": True}),
            response_status_code=True,
        )

        status_code: int = api_call.get("status")

        alert_manager_status_dict: dict = dict(
            {
                500: "Failed to reset basic roles.",
            }
        )

        if status_code == 200:
            if "Reset performed" != api_call.get("message"):
                logging.error(f"Please, check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully performed the reset.")
        elif 403 <= status_code <= 500:
            logging.error(alert_manager_status_dict.get(status_code))
            raise Exception
        else:
            logging.error(f"Check the error: {api_call}.")
            raise Exception



import json
from httpx import Response
import logging

from .model import (
    APIModel,
    APIEndpoints,
    RequestsMethods,
)
from .api import Api


class Licensing:
    """The class includes all necessary methods to access the Grafana licensing API endpoints. Be aware that the functionality is a Grafana ENTERPRISE v7.4+ feature

    HINT: Note Grafana Enterprise API need required permissions if fine-grained access control is enabled

    Args:
        grafana_api_model (APIModel): Inject a Grafana API model object that includes all necessary values and information

    Attributes:
        grafana_api_model (APIModel): This is where we store the grafana_api_model
    """

    def __init__(self, grafana_api_model: APIModel):
        self.grafana_api_model = grafana_api_model

    def check_license_availability(self):
        """The method includes a functionality to checks if a valid license is available

        Required Permissions:
            Action: licensing:read
            Scope: N/A

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (bool): Returns the result if the license is available or not
        """

        api_call: Response = Api(self.grafana_api_model).call_the_api(
            f"{APIEndpoints.LICENSING.value}/check",
        )

        if api_call.status_code != 200:
            logging.error(f"Check the error: {api_call}.")
            raise Exception
        else:
            return json.loads(str(api_call.text))

    def manually_force_license_refresh(self):
        """The method includes a functionality to manually ask license issuer for a new token

        Required Permissions:
            Action: licensing:update
            Scope: N/A

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the result of license refresh call
        """

        api_call: dict = Api(self.grafana_api_model).call_the_api(
            f"{APIEndpoints.LICENSING.value}/token/renew",
            RequestsMethods.POST,
            json.dumps({}),
        )

        if api_call == dict() or api_call.get("jti") is None:
            logging.error(f"Check the error: {api_call}.")
            raise Exception
        else:
            return api_call

    def remove_license_from_database(self):
        """The method includes a functionality to removes the license stored in the Grafana database

        Required Permissions:
            Action: licensing:delete
            Scope: N/A

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the result of license refresh call
        """

        api_call: dict = Api(self.grafana_api_model).call_the_api(
            f"{APIEndpoints.LICENSING.value}/token",
            RequestsMethods.DELETE,
            response_status_code=True,
        )

        if api_call.get("status") != 200:
            logging.error(f"Check the error: {api_call}.")
            raise Exception
        else:
            logging.info(
                "You successfully removed the corresponding license from the database."
            )



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    coverage_string: str = "![Coverage report](https://github.com/ZPascal/grafana_api_sdk/blob/main/docs/coverage.svg)"
    long_description: str = fh.read()

long_description = long_description.replace(coverage_string, "")

setuptools.setup(
    name="grafana-api-sdk",
    version="0.7.2",
    author="Pascal Zimmermann",
    author_email="info@theiotstudio.com",
    description="A Grafana API SDK",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/ZPascal/grafana_api_sdk",
    project_urls={
        "Source": "https://github.com/ZPascal/grafana_api_sdk",
        "Bug Tracker": "https://github.com/ZPascal/grafana_api_sdk/issues",
        "Documentation": "https://zpascal.github.io/grafana_api_sdk/",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved",
        "Operating System :: OS Independent",
    ],
    license="Apache-2.0 License",
    packages=["grafana_api"],
    install_requires=["httpx"],
    extras_require={
        "http2": ["httpx[http2]"],
    },
    tests_require=["pytest-httpx", "pytest"],
    python_requires=">=3.8",
)



import json
import logging

from .model import (
    APIModel,
    APIEndpoints,
    RequestsMethods,
    PlaylistObject,
)
from .api import Api


class Playlist:
    """The class includes all necessary methods to access the Grafana playlist API endpoints

    Args:
        grafana_api_model (APIModel): Inject a Grafana API model object that includes all necessary values and information

    Attributes:
        grafana_api_model (APIModel): This is where we store the grafana_api_model
    """

    def __init__(self, grafana_api_model: APIModel):
        self.grafana_api_model = grafana_api_model

    def search_playlist(self, query: str = None, limit: int = None) -> list:
        """The method includes a functionality to get the organization playlist's specified by the optional pagination functionality

         Args:
            query (str): Specify the query to limit response to playlist having a name like this value(default None)
            limit (int): Specify the limit as integer of the response (default None)

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (list): Returns the organization playlist's
        """

        api_request_url: str = APIEndpoints.PLAYLISTS.value

        if query is not None:
            api_request_url: str = f"{api_request_url}?query={query}"

        if limit is not None and limit != 0:
            api_request_url: str = f"{api_request_url}?limit={limit}"

        if query is not None and (limit is not None or limit != 0):
            api_request_url: str = f"{api_request_url}?query={query}&limit={limit}"

        api_call: list = Api(self.grafana_api_model).call_the_api(
            api_request_url,
        )

        if api_call == list() or api_call[0].get("uid") is None:
            logging.error(f"Check the error: {api_call}.")
            raise Exception
        else:
            return api_call

    def get_playlist(self, playlist_uid: str) -> dict:
        """The method includes a functionality to get the playlist specified by the playlist_uid

         Args:
            playlist_uid (str): Specify the playlist_uid

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the corresponding playlist
        """

        if len(playlist_uid) != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.PLAYLISTS.value}/{playlist_uid}",
            )

            if api_call == dict() or api_call.get("uid") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no playlist_uid defined.")
            raise ValueError

    def get_playlist_items(self, playlist_uid: str) -> list:
        """The method includes a functionality to get the playlist items specified by the playlist_uid

         Args:
            playlist_uid (str): Specify the playlist_uid

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the corresponding playlist items
        """

        if len(playlist_uid) != 0:
            api_call: list = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.PLAYLISTS.value}/{playlist_uid}/items",
            )

            if api_call == list() or api_call[0].get("value") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no playlist_uid defined.")
            raise ValueError

    def get_playlist_dashboards(self, playlist_uid: str) -> list:
        """The method includes a functionality to get the playlist dashboards specified by the playlist_uid

         Args:
            playlist_uid (str): Specify the playlist_uid

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the corresponding playlist dashboards
        """

        if len(playlist_uid) != 0:
            api_call: list = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.PLAYLISTS.value}/{playlist_uid}/dashboards",
            )

            if api_call == list() or api_call[0].get("title") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no playlist_uid defined.")
            raise ValueError

    def create_playlist(self, playlist: PlaylistObject) -> dict:
        """The method includes a functionality to create a playlist specified by the playlist object

         Args:
            playlist (PlaylistObject): Specify the playlist object

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the corresponding playlist
        """

        if playlist is not None:
            items: list = list()

            for item in playlist.items:
                items.append(
                    {
                        "type": item.type,
                        "value": item.value,
                        "order": item.order,
                        "title": item.title,
                    }
                )

            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.PLAYLISTS.value}",
                RequestsMethods.POST,
                json.dumps(
                    dict(
                        {
                            "name": playlist.name,
                            "interval": playlist.interval,
                            "items": items,
                        }
                    )
                ),
            )

            if api_call == dict() or api_call.get("uid") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no playlist object defined.")
            raise ValueError

    def update_playlist(self, playlist_uid: str, playlist: PlaylistObject) -> dict:
        """The method includes a functionality to update a playlist specified by the playlist object and playlist_uid

         Args:
            playlist_uid (str): Specify the playlist_uid
            playlist (PlaylistObject): Specify the playlist object

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the corresponding playlist
        """

        if len(playlist_uid) != 0 and playlist is not None:
            items: list = list()

            for item in playlist.items:
                items.append(
                    {
                        "type": item.type,
                        "value": item.value,
                        "order": item.order,
                        "title": item.title,
                    }
                )

            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.PLAYLISTS.value}/{playlist_uid}",
                RequestsMethods.PUT,
                json.dumps(
                    dict(
                        {
                            "name": playlist.name,
                            "interval": playlist.interval,
                            "items": items,
                        }
                    )
                ),
            )

            if api_call == dict() or api_call.get("uid") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no playlist_uid or playlist object defined.")
            raise ValueError

    def delete_playlist(self, playlist_uid: str):
        """The method includes a functionality to delete a playlist specified by the playlist_uid

         Args:
            playlist_uid (str): Specify the playlist_uid

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if len(playlist_uid) != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.PLAYLISTS.value}/{playlist_uid}",
                RequestsMethods.DELETE,
                response_status_code=True,
            )

            if api_call.get("status") != 200:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully deleted the corresponding playlist.")
        else:
            logging.error("There is no playlist_uid object defined.")
            raise ValueError



import json
import logging

from .model import (
    APIModel,
    APIEndpoints,
    RequestsMethods,
    AnnotationObject,
    AnnotationGraphiteObject,
    FindAnnotationObject,
)
from .api import Api


class Annotations:
    """The class includes all necessary methods to access the Grafana annotations API endpoints. Annotations can be organization annotations that can be shown on any dashboard by configuring an annotation data source filtered by tags. They can also be tied to a panel on a dashboard and are then only shown on that panel

    HINT: Note Grafana Enterprise API need required permissions if fine-grained access control is enabled

    Args:
        grafana_api_model (APIModel): Inject a Grafana API model object that includes all necessary values and information

    Attributes:
        grafana_api_model (APIModel): This is where we store the grafana_api_model
    """

    def __init__(self, grafana_api_model: APIModel):
        self.grafana_api_model = grafana_api_model

    def find_annotations(self, annotation: FindAnnotationObject = None) -> list:
        """The method includes a functionality to find the corresponding annotations

        Args:
            annotation (FindAnnotationObject): Specify the find annotation object

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (list): Returns the result of the find annotations call
        """

        custom_query: str = "?"

        if annotation is not None:
            if annotation.from_value is not None and annotation.from_value != 0:
                custom_query = f"from={annotation.from_value}"

            if annotation.to_value is not None and annotation.to_value != 0:
                custom_query = Api.prepare_api_string(custom_query)
                custom_query = f"{custom_query}to={annotation.to_value}"

            if annotation.limit != 100 and annotation.limit != 0:
                custom_query = Api.prepare_api_string(custom_query)
                custom_query = f"{custom_query}limit={annotation.limit}"

            if annotation.alert_id is not None and annotation.alert_id != 0:
                custom_query = Api.prepare_api_string(custom_query)
                custom_query = f"{custom_query}alertId={annotation.alert_id}"

            if annotation.dashboard_id is not None and annotation.dashboard_id != 0:
                custom_query = Api.prepare_api_string(custom_query)
                custom_query = f"{custom_query}dashboardId={annotation.dashboard_id}"

            if annotation.panel_id is not None and annotation.panel_id != 0:
                custom_query = Api.prepare_api_string(custom_query)
                custom_query = f"{custom_query}panelId={annotation.panel_id}"

            if annotation.user_id is not None and annotation.user_id != 0:
                custom_query = Api.prepare_api_string(custom_query)
                custom_query = f"{custom_query}userId={annotation.user_id}"

            if annotation.type is not None and len(annotation.type) != 0:
                custom_query = Api.prepare_api_string(custom_query)
                custom_query = f"{custom_query}type={annotation.type}"

            if annotation.tags is not None:
                custom_query = Api.prepare_api_string(custom_query)

                tags_string: str = ""
                for i in range(0, len(annotation.tags)):
                    tags_string = f"{tags_string}tags={annotation.tags[i]}"

                    if i < len(annotation.tags) - 1:
                        tags_string = f"{tags_string}&"

                custom_query = f"{custom_query}{tags_string}"

        if len(custom_query) == 1:
            custom_query = ""

        api_call: list = Api(self.grafana_api_model).call_the_api(
            f"{APIEndpoints.ANNOTATIONS.value}{custom_query}",
        )

        if api_call == list() or api_call[0].get("id") is None:
            logging.error(f"Check the error: {api_call}.")
            raise Exception
        else:
            return api_call

    def create_annotation(
        self,
        annotation: AnnotationObject,
    ) -> int:
        """The method includes a functionality to create the corresponding annotation

        Args:
            annotation (AnnotationObject): Specify the annotation object

        Required Permissions:
            Action: annotations:create
            Scope: annotations:type:

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (int): Returns the annotation id
        """

        if (
            annotation is not None
            and annotation.time != 0
            and annotation.time_end != 0
            and len(annotation.text) != 0
            and len(annotation.tags) != 0
        ):
            annotation_object: dict = dict(
                {
                    "time": annotation.time,
                    "timeEnd": annotation.time_end,
                    "tags": annotation.tags,
                    "text": annotation.text,
                }
            )

            if annotation.dashboard_uid is not None:
                annotation_object.update(
                    dict({"dashboardUID": annotation.dashboard_uid})
                )

            if annotation.panel_id is not None:
                annotation_object.update(dict({"panelId": annotation.panel_id}))

            api_call: dict = Api(self.grafana_api_model).call_the_api(
                APIEndpoints.ANNOTATIONS.value,
                RequestsMethods.POST,
                json.dumps(annotation_object),
            )

            if (
                api_call.get("message") != "Annotation added"
                or api_call.get("id") is None
            ):
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return int(api_call.get("id"))
        else:
            logging.error("There is no time, time_end, text or tags defined.")
            raise ValueError

    def create_graphite_annotation(self, annotation: AnnotationGraphiteObject) -> int:
        """The method includes a functionality to create the corresponding graphite annotation

        Args:
            annotation (AnnotationGraphiteObject): Specify the annotation object

        Required Permissions:
            Action: annotations:create
            Scope: annotations:type:organization

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (int): Returns the annotation id
        """

        if (
            annotation is not None
            and len(annotation.what) != 0
            and len(annotation.tags) != 0
        ):
            annotation_object: dict = dict(
                {"what": annotation.what, "tags": annotation.tags}
            )

            if annotation.when is not None:
                annotation_object.update(dict({"when": annotation.when}))

            if annotation.data is not None:
                annotation_object.update(dict({"data": annotation.data}))

            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.ANNOTATIONS.value}/graphite",
                RequestsMethods.POST,
                json.dumps(annotation_object),
            )

            if (
                api_call.get("message") != "Graphite annotation added"
                or api_call.get("id") is None
            ):
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return int(api_call.get("id"))
        else:
            logging.error("There is no what or tags defined.")
            raise ValueError

    def update_annotation(self, id: int, annotation: AnnotationObject):
        """The method includes a functionality to update the corresponding annotation specified by the annotation id

        Args:
            id (int): Specify the annotation object id
            annotation (AnnotationObject): Specify the annotation object

        Required Permissions:
            Action: annotations:write
            Scope: annotations:type:

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if id != 0 and annotation is not None:
            annotation_object: dict = dict()

            if annotation.time is not None and annotation.time != 0:
                annotation_object.update(dict({"time": annotation.time}))

            if annotation.time_end is not None and annotation.time_end != 0:
                annotation_object.update(dict({"timeEnd": annotation.time_end}))

            if annotation.text is not None and len(annotation.text) != 0:
                annotation_object.update(dict({"text": annotation.text}))

            if annotation.tags is not None and len(annotation.tags) != 0:
                annotation_object.update(dict({"text": annotation.text}))

            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.ANNOTATIONS.value}/{id}",
                RequestsMethods.PATCH,
                json.dumps(annotation_object),
            )

            if api_call.get("message") != "Annotation patched":
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully updated the annotation.")
        else:
            logging.error("There is no id defined.")
            raise ValueError

    def delete_annotation(
        self,
        id: int,
    ):
        """The method includes a functionality to delete the corresponding annotation specified by the annotation id

        Args:
            id (int): Specify the annotation object id

        Required Permissions:
            Action: annotations:write
            Scope: annotations:type:

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if id != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.ANNOTATIONS.value}/{id}",
                RequestsMethods.DELETE,
            )

            if api_call.get("message") != "Annotation deleted":
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully deleted the annotation.")
        else:
            logging.error("There is no id defined.")
            raise ValueError

    def find_annotation_tags(
        self,
        tag: str = None,
        limit: int = 100,
    ):
        """The method includes a functionality to find the annotation tags

        Args:
            tag (str): Specify the optional annotation tag
            limit (int): Specify the optional annotation limit (default 100)

        Required Permissions:
            Action: annotations:read
            Scope: N/A

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the result of the find annotation tags call
        """

        filter_values: str = ""

        if tag is not None and len(tag) != 0:
            filter_values = f"?tag={tag}"

        if limit != 100:
            if tag is None:
                filter_values = "?"

            if len(filter_values) == 1:
                filter_values = f"{filter_values}limit={limit}"
            else:
                filter_values = f"{filter_values}&limit={limit}"

        api_call: dict = Api(self.grafana_api_model).call_the_api(
            f"{APIEndpoints.ANNOTATIONS.value}/tags{filter_values}",
        )

        if api_call == dict() or api_call.get("result") is None:
            logging.error(f"Check the error: {api_call}.")
            raise Exception
        else:
            return api_call



import json
import logging

from .model import APIModel, APIEndpoints, RequestsMethods, PublicDashboard
from .folder import Folder
from .api import Api


class Dashboard:
    """The class includes all necessary methods to access the Grafana dashboard API endpoints

    Args:
        grafana_api_model (APIModel): Inject a Grafana API model object that includes all necessary values and information

    Attributes:
        grafana_api_model (APIModel): This is where we store the grafana_api_model
    """

    def __init__(self, grafana_api_model: APIModel):
        self.grafana_api_model = grafana_api_model

    def create_or_update_dashboard(
        self,
        dashboard_path: str,
        dashboard_json: dict,
        message: str,
        overwrite: bool = False,
    ):
        """The method includes a functionality to create the specified dashboard

        Args:
            dashboard_path (str): Specify the dashboard path in which the dashboard is to be placed
            dashboard_json (dict): Specify the inserted dashboard as dict
            message (str): Specify the message that should be injected as commit message inside the dashboard
            overwrite (bool): Should the already existing dashboard be overwritten (default False)

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if len(dashboard_path) != 0 and dashboard_json != dict() and len(message) != 0:
            folder_id: int = Folder(
                self.grafana_api_model
            ).get_folder_id_by_dashboard_path(dashboard_path)

            dashboard_json_complete: dict = {
                "dashboard": dashboard_json,
                "folderId": folder_id,
                "message": message,
                "overwrite": overwrite,
            }

            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.DASHBOARDS.value}/db",
                RequestsMethods.POST,
                json.dumps(dashboard_json_complete),
            )

            if api_call.get("status") != "success":
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully deployed the dashboard.")
        else:
            logging.error(
                "There is no dashboard_path or dashboard_json or message defined."
            )
            raise ValueError

    def delete_dashboard_by_name_and_path(
        self, dashboard_name: str, dashboard_path: str
    ):
        """The method includes a functionality to delete the specified dashboard inside the model

        Args:
            dashboard_name (str): Specify the dashboard name of the deleted dashboard
            dashboard_path (str): Specify the dashboard path in which the dashboard is to be placed

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if len(dashboard_name) != 0 and len(dashboard_path) != 0:
            dashboard_uid: dict = self.get_dashboard_uid_and_id_by_name_and_folder(
                dashboard_name, dashboard_path
            )

            if len(dashboard_uid) != 0:
                api_call: dict = Api(self.grafana_api_model).call_the_api(
                    f"{APIEndpoints.DASHBOARDS.value}/uid/{dashboard_uid.get('uid')}",
                    RequestsMethods.DELETE,
                )

                if f"Dashboard {dashboard_name} deleted" != api_call.get("message"):
                    logging.error(f"Please, check the error: {api_call}.")
                    raise Exception
                else:
                    logging.info("You successfully destroyed the dashboard.")
            else:
                logging.error("Nothing to delete. There is no dashboard available.")
                raise ValueError
        else:
            logging.error("There is no dashboard_name or dashboard_path defined.")
            raise ValueError

    def get_dashboard_by_uid(self, uid: str) -> dict:
        """The method includes a functionality to get the dashboard from the specified uid

        Args:
            uid (str): Specify the uid of the dashboard

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the dashboard
        """

        if len(uid) != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.DASHBOARDS.value}/uid/{uid}"
            )

            if api_call.get("dashboard") is None:
                logging.error(f"Please, check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no dashboard uid defined.")
            raise ValueError

    def get_dashboard_home(self) -> dict:
        """The method includes a functionality to get the home dashboard

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the home dashboard
        """

        api_call: dict = Api(self.grafana_api_model).call_the_api(
            f"{APIEndpoints.DASHBOARDS.value}/home"
        )

        if api_call.get("dashboard") is None:
            logging.error(f"Please, check the error: {api_call}.")
            raise Exception
        else:
            return api_call

    def get_dashboard_tags(self) -> list:
        """The method includes a functionality to get the all tags of all dashboards

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (list): Returns all dashboard tags
        """

        api_call: list = Api(self.grafana_api_model).call_the_api(
            f"{APIEndpoints.DASHBOARDS.value}/tags"
        )

        if api_call == list() or api_call[0].get("term") is None:
            logging.error(f"Please, check the error: {api_call}.")
            raise Exception
        else:
            return api_call

    def get_dashboard_uid_and_id_by_name_and_folder(
        self, dashboard_name: str, dashboard_path: str
    ) -> dict:
        """The method includes a functionality to extract the dashboard uid specified inside the model

        Args:
            dashboard_name (str): Specify the dashboard name of the dashboard
            dashboard_path (str): Specify the dashboard path of the dashboard

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the dashboard uid and the id
        """

        if len(dashboard_name) != 0 and len(dashboard_path) != 0:
            folder_id: int = Folder(
                self.grafana_api_model
            ).get_folder_id_by_dashboard_path(dashboard_path)

            search_query: str = (
                f"{APIEndpoints.SEARCH.value}?folderIds={folder_id}&query={dashboard_name}"
            )
            dashboard_meta: list = Api(self.grafana_api_model).call_the_api(
                search_query
            )

            for dashboard_meta_object in dashboard_meta:
                if dashboard_meta_object.get("title") is not None:
                    if dashboard_meta_object.get("title") == dashboard_name:
                        if (
                            dashboard_meta_object.get("uid") is not None
                            and dashboard_meta_object.get("id") is not None
                        ):
                            return dict(
                                {
                                    "uid": dashboard_meta_object.get("uid"),
                                    "id": dashboard_meta_object.get("id"),
                                }
                            )
                        else:
                            logging.error("There is no uid or id defined.")
                            raise ValueError
                else:
                    logging.error("There is no title defined.")
                    raise ValueError
        else:
            logging.error("There is no dashboard_name or dashboard_path defined.")
            raise ValueError

    def get_dashboard_permissions(self, id: int) -> list:
        """The method includes a functionality to extract the dashboard permissions based on the specified id

        Args:
            id (int): Specify the id of the dashboard

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (list): Returns the dashboard permissions of a dashboard as list
        """

        if id != 0:
            api_call: list = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.DASHBOARDS.value}/id/{id}/permissions"
            )

            if api_call == list() or api_call[0].get("role") is None:
                logging.error(f"Please, check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no dashboard id defined.")
            raise ValueError

    def get_dashboard_permissions_by_uid(self, uid: str) -> list:
        """The method includes a functionality to extract the dashboard permissions based on the specified uid

        Args:
            uid (str): Specify the uid of the dashboard

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (list): Returns the dashboard permissions of a dashboard as list
        """

        if len(uid) != 0:
            api_call: list = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.DASHBOARDS.value}/uid/{uid}/permissions"
            )

            if api_call == list() or api_call[0].get("role") is None:
                logging.error(f"Please, check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no dashboard uid defined.")
            raise ValueError

    def update_dashboard_permissions(self, id: int, permission_json: dict):
        """The method includes a functionality to update the dashboard permissions based on the specified id and the permission json document

        Args:
            id (int): Specify the id of the dashboard
            permission_json (dict): Specify the inserted permissions as dict

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if id != 0 and len(permission_json) != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.DASHBOARDS.value}/id/{id}/permissions",
                RequestsMethods.POST,
                json.dumps(permission_json),
            )

            if api_call.get("message") != "Dashboard permissions updated":
                logging.error(f"Please, check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully modified the dashboard permissions.")
        else:
            logging.error("There is no dashboard id or permission json defined.")
            raise ValueError

    def update_dashboard_permissions_by_uid(self, uid: str, permission_json: dict):
        """The method includes a functionality to update the dashboard permissions based on the specified uid and the permission json document

        Args:
            uid (str): Specify the uid of the dashboard
            permission_json (dict): Specify the inserted permissions as dict

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if len(uid) != 0 and len(permission_json) != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.DASHBOARDS.value}/uid/{uid}/permissions",
                RequestsMethods.POST,
                json.dumps(permission_json),
            )

            if api_call.get("message") != "Dashboard permissions updated":
                logging.error(f"Please, check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully modified the dashboard permissions.")
        else:
            logging.error("There is no dashboard uid or permission json defined.")
            raise ValueError

    def get_dashboard_versions(self, id: int) -> list:
        """The method includes a functionality to extract the versions of a dashboard based on the specified id

        Args:
            id (int): Specify the id of the dashboard

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (list): Returns all dashboard versions of a dashboard as list
        """

        if id != 0:
            api_call: list = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.DASHBOARDS.value}/id/{id}/versions",
            )

            if api_call == list() or api_call[0].get("id") is None:
                logging.error(f"Please, check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no dashboard id defined.")
            raise ValueError

    def get_dashboard_versions_by_uid(self, uid: str) -> list:
        """The method includes a functionality to extract the versions of a dashboard based on the specified uid

        Args:
            uid (str): Specify the id of the dashboard

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (list): Returns all dashboard versions of a dashboard as list
        """

        if len(uid) != 0:
            api_call: list = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.DASHBOARDS.value}/uid/{uid}/versions",
            )

            if api_call == list() or api_call[0].get("uid") is None:
                logging.error(f"Please, check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no dashboard uid defined.")
            raise ValueError

    def get_dashboard_version(self, id: int, version_id: int) -> dict:
        """The method includes a functionality to extract a specified version of a dashboard based on the specified dashboard id and a version_id of the dashboard

        Args:
            id (int): Specify the id of the dashboard
            version_id (int): Specify the version_id of a dashboard

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (list): Returns a dashboard version of a dashboard as dict
        """

        if id != 0 and version_id != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.DASHBOARDS.value}/id/{id}/versions/{version_id}",
            )

            if api_call == dict() or api_call.get("id") is None:
                logging.error(f"Please, check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no dashboard id or version_id defined.")
            raise ValueError

    def get_dashboard_version_by_uid(self, uid: str, version_id: int) -> dict:
        """The method includes a functionality to extract a specified version of a dashboard based on the specified dashboard uid and a version_id of the dashboard

        Args:
            uid (str): Specify the uid of the dashboard
            version_id (int): Specify the version_id of a dashboard

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (list): Returns a dashboard version of a dashboard as dict
        """

        if len(uid) != 0 and version_id != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.DASHBOARDS.value}/uid/{uid}/versions/{version_id}",
            )

            if api_call == dict() or api_call.get("uid") is None:
                logging.error(f"Please, check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no dashboard uid or version_id defined.")
            raise ValueError

    def restore_dashboard_version(self, id: int, version: dict):
        """The method includes a functionality to restore a specified version of a dashboard based on the specified dashboard id and a version as dict of the dashboard

        Args:
            id (int): Specify the id of the dashboard
            version (dict): Specify the version_id of a dashboard

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if id != 0 and version != dict():
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.DASHBOARDS.value}/id/{id}/restore",
                RequestsMethods.POST,
                json.dumps(version),
            )

            if (
                api_call.get("status") != "success"
                or api_call.get("message") is not None
            ):
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully restored the dashboard.")
        else:
            logging.error("There is no dashboard id or version_id defined.")
            raise ValueError

    def restore_dashboard_version_by_uid(self, uid: str, version: dict):
        """The method includes a functionality to restore a specified version of a dashboard based on the specified dashboard uid and a version as dict of the dashboard

        Args:
            uid (str): Specify the uid of the dashboard
            version (dict): Specify the version_id of a dashboard

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if len(uid) != 0 and version != dict():
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.DASHBOARDS.value}/uid/{uid}/restore",
                RequestsMethods.POST,
                json.dumps(version),
            )

            if (
                api_call.get("status") != "success"
                or api_call.get("message") is not None
            ):
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully restored the dashboard.")
        else:
            logging.error("There is no dashboard uid or version_id defined.")
            raise ValueError

    def calculate_dashboard_diff(
        self,
        dashboard_id_and_version_base: dict,
        dashboard_id_and_version_new: dict,
        diff_type: str = "json",
    ) -> str:
        """The method includes a functionality to calculate the diff of specified versions of a dashboard based on the specified dashboard uid and the selected version of the base dashboard and the new dashboard and the diff type (basic or json)

        Args:
            dashboard_id_and_version_base (dict): Specify the version and id of the base dashboard
            dashboard_id_and_version_new (dict): Specify the version and id of the new dashboard
            diff_type (str): Specify the diff type (basic or json) (default json)

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (str): Returns the difference of the two specified dashboards
        """
        possible_diff_types: list = list(["basic", "json"])

        if diff_type.lower() in possible_diff_types:
            if (
                dashboard_id_and_version_base != dict()
                and dashboard_id_and_version_new != 0
            ):
                diff_object: dict = dict()
                diff_object.update({"base": dashboard_id_and_version_base})
                diff_object.update({"new": dashboard_id_and_version_new})
                diff_object.update({"diffType": diff_type.lower()})

                api_call: any = Api(self.grafana_api_model).call_the_api(
                    f"{APIEndpoints.DASHBOARDS.value}/calculate-diff",
                    RequestsMethods.POST,
                    json.dumps(diff_object),
                )

                if api_call.status_code != 200:
                    logging.error(f"Check the error: {api_call.text}.")
                    raise Exception
                else:
                    return api_call.text
            else:
                logging.error(
                    "There is no dashboard_uid_and_version_base or dashboard_uid_and_version_new defined."
                )
                raise ValueError
        else:
            logging.error(
                f"The diff_type: {diff_type.lower()} is not valid. Please specify a valid value."
            )
            raise ValueError

    def get_public_dashboards(self, per_page: int = None, page: int = None) -> dict:
        """The method includes a functionality to get all public available dashboards

        Required Permissions:
            Action: dashboards:read
            Scope: dashboards:uid:<dashboard UID>

        Args:
            per_page (int): Specify the value per page size
            page (int): Specify the page

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns all public available dashboards
        """

        optional_parts: str = "?"

        if per_page is not None:
            optional_parts += f"perpage={per_page}"

        if page is not None:
            if len(optional_parts) > 1:
                optional_parts += "&"

            optional_parts += f"page={page}"

        if len(optional_parts) == 1:
            optional_parts: str = ""

        api_call: dict = Api(self.grafana_api_model).call_the_api(
            f"{APIEndpoints.DASHBOARDS.value}/public-dashboards{optional_parts}",
        )

        if isinstance(api_call, dict) is False or api_call == dict():
            logging.error(f"Check the error: {api_call}.")
            raise Exception
        else:
            return api_call

    def get_public_dashboard_by_uid(
        self,
        dashboard_uid: str,
    ) -> dict:
        """The method includes a functionality to get a public available dashboard specified by dashboard_uid

        Required Permissions:
            Action: dashboards:read
            Scope: dashboards:uid:<dashboard UID>

        Args:
            dashboard_uid (str): Specify the dashboard_uid

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the corresponding public available dashboard
        """

        if dashboard_uid is not None and len(dashboard_uid) != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.DASHBOARDS.value}/uid/{dashboard_uid}/public-dashboards",
            )

            if isinstance(api_call, dict) is False or api_call == dict():
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no dashboard uid defined.")
            raise ValueError

    def create_public_dashboard(
        self, dashboard_uid: str, public_dashboard: PublicDashboard = PublicDashboard()
    ) -> dict:
        """The method includes a functionality to create a public available dashboard

        Required Permissions:
            Action: dashboards.public:write
            Scope: dashboards:uid:<dashboard UID>

        Args:
            dashboard_uid (str): Specify the dashboard_uid
            public_dashboard (PublicDashboard): Specify the optional public dashboard object

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the corresponding public available dashboard
        """

        if (
            dashboard_uid is not None
            and len(dashboard_uid) != 0
            and public_dashboard is not None
        ):
            public_dashboard_result: dict = dict(
                {
                    "uid": public_dashboard.uid,
                    "accessToken": public_dashboard.access_token,
                    "timeSelectionEnabled": public_dashboard.time_selection_enabled,
                    "isEnabled": public_dashboard.is_enabled,
                    "annotationsEnabled": public_dashboard.annotations_enabled,
                    "share": public_dashboard.share,
                }
            )

            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.DASHBOARDS.value}/uid/{dashboard_uid}/public-dashboards",
                RequestsMethods.POST,
                json.dumps(public_dashboard_result),
                response_status_code=True,
            )

            status_code: int = api_call.get("status")
            public_dashboard_status_dict: dict = dict(
                {
                    400: "Dashboard is already public.",
                    401: "Unauthorized.",
                    403: "Access denied.",
                    404: "Dashboard not found.",
                }
            )

            if status_code == 200 and (
                isinstance(api_call, dict) is True or api_call != dict()
            ):
                return api_call
            elif 400 <= status_code <= 404:
                logging.error(public_dashboard_status_dict.get(status_code))
                raise Exception
        else:
            logging.error("There is no dashboard uid or public dashboard defined.")
            raise ValueError

    def update_public_dashboard(
        self,
        dashboard_uid: str,
        public_dashboard_uid: str,
        time_selection_enabled: bool = None,
        is_enabled: bool = None,
        annotations_enabled: bool = None,
        share: str = None,
    ) -> dict:
        """The method includes a functionality to update a public available dashboard

        Required Permissions:
            Action: dashboards.public:write
            Scope: dashboards:uid:<dashboard UID>

        Args:
            dashboard_uid (str): Specify the dashboard_uid
            public_dashboard_uid (str): Specify the public_dashboard_uid
            time_selection_enabled (bool): Specify the optional enablement of the time picker inside the public dashboard (default None)
            is_enabled (bool): Specify the optional enablement of the public dashboard (default None)
            annotations_enabled (bool): Specify the optional enablement of the annotations inside the public dashboard (default None)
            share (str): Specify the optional share mode of the public dashboard (default None)

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the corresponding public available dashboard
        """

        if (
            dashboard_uid is not None
            and len(dashboard_uid) != 0
            and public_dashboard_uid is not None
            and len(public_dashboard_uid) != 0
        ):
            public_dashboard_result: dict = dict()

            if time_selection_enabled is not None:
                public_dashboard_result.update(
                    {"timeSelectionEnabled": time_selection_enabled}
                )

            if is_enabled is not None:
                public_dashboard_result.update({"isEnabled": is_enabled})

            if annotations_enabled is not None:
                public_dashboard_result.update(
                    {"annotationsEnabled": annotations_enabled}
                )

            if share is not None:
                public_dashboard_result.update({"share": share})

            if public_dashboard_result == dict():
                logging.error(
                    "There is no values for the update of the public dashboard defined."
                )
                raise ValueError

            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.DASHBOARDS.value}/uid/{dashboard_uid}/public-dashboards/{public_dashboard_uid}",
                RequestsMethods.PATCH,
                json.dumps(public_dashboard_result),
                response_status_code=True,
            )

            status_code: int = api_call.get("status")
            public_dashboard_status_dict: dict = dict(
                {
                    400: "Dashboard is already public.",
                    401: "Unauthorized.",
                    403: "Access denied.",
                    404: "Public dashboard not found.",
                }
            )

            if status_code == 200 and (
                isinstance(api_call, dict) is True or api_call != dict()
            ):
                return api_call
            elif 400 <= status_code <= 404:
                logging.error(public_dashboard_status_dict.get(status_code))
                raise Exception
        else:
            logging.error("There is no dashboard uid or public dashboard defined.")
            raise ValueError

    def delete_public_dashboard(
        self,
        dashboard_uid: str,
        public_dashboard_uid: str,
    ):
        """The method includes a functionality to delete a public available dashboard

        Required Permissions:
            Action: dashboards.public:write
            Scope: dashboards:uid:<dashboard UID>

        Args:
            dashboard_uid (str): Specify the dashboard_uid
            public_dashboard_uid (str): Specify the public_dashboard_uid

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if (
            dashboard_uid is not None
            and len(dashboard_uid) != 0
            and public_dashboard_uid is not None
            and len(public_dashboard_uid) != 0
        ):
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.DASHBOARDS.value}/uid/{dashboard_uid}/public-dashboards/{public_dashboard_uid}",
                RequestsMethods.DELETE,
                response_status_code=True,
            )

            status_code: int = api_call.get("status")
            public_dashboard_status_dict: dict = dict(
                {
                    401: "Unauthorized.",
                    403: "Access denied.",
                }
            )

            if status_code == 200:
                logging.info("You successfully deleted the public dashboard.")
            elif 401 <= status_code <= 403:
                logging.error(public_dashboard_status_dict.get(status_code))
                raise Exception
        else:
            logging.error("There is no dashboard uid or public dashboard defined.")
            raise ValueError
